@@ -4,13 +4,13 @@
 import argparse
 import shutil, re
 from pathlib import Path, PurePath
+from biobb_common.generic.biobb_object import BiobbObject
 from biobb_common.configuration import  settings
 from biobb_common.tools import file_utils as fu
 from biobb_common.tools.file_utils import launchlogger
-from biobb_common.command_wrapper import cmd_wrapper
 from biobb_amber.sander.common import *
 
-class SanderMDRun():
+class SanderMDRun(BiobbObject):
     """
     | biobb_amber SanderMDRun
     | Wrapper of the `AmberTools (AMBER MD Package) sander tool <https://ambermd.org/AmberTools.php>`_ module.
@@ -18,13 +18,13 @@ class SanderMDRun():
 
     Args:
         input_top_path (str): Input topology file (AMBER ParmTop). File type: input. `Sample file <https://github.com/bioexcel/biobb_amber/raw/master/biobb_amber/test/data/sander/cln025.prmtop>`_. Accepted formats: top (edam:format_3881), parmtop (edam:format_3881), prmtop (edam:format_3881).
-        input_crd_path (str): Input coordinates file (AMBER crd). File type: input. `Sample file <https://github.com/bioexcel/biobb_amber/raw/master/biobb_amber/test/data/sander/cln025.inpcrd>`_. Accepted formats: crd (edam:format_3878), mdcrd (edam:format_3878), inpcrd (edam:format_3878), netcdf (edam:format_3650), nc (edam:format_3650), ncrst (edam:format_3650).
+        input_crd_path (str): Input coordinates file (AMBER crd). File type: input. `Sample file <https://github.com/bioexcel/biobb_amber/raw/master/biobb_amber/test/data/sander/cln025.inpcrd>`_. Accepted formats: crd (edam:format_3878), mdcrd (edam:format_3878), inpcrd (edam:format_3878), netcdf (edam:format_3650), nc (edam:format_3650), ncrst (edam:format_3886).
         input_mdin_path (str) (Optional): Input configuration file (MD run options) (AMBER mdin). File type: input. `Sample file <https://github.com/bioexcel/biobb_amber/raw/master/biobb_amber/test/data/sander/npt.mdin>`_. Accepted formats: mdin (edam:format_2330), in (edam:format_2330), txt (edam:format_2330).
         input_cpin_path (str) (Optional): Input constant pH file (AMBER cpin). File type: input. `Sample file <https://github.com/bioexcel/biobb_amber/raw/master/biobb_amber/test/data/sander/cln025.cpin>`_. Accepted formats: cpin (edam:format_2330).
-        input_ref_path (str) (Optional): Input reference coordinates for position restraints. File type: input. `Sample file <https://github.com/bioexcel/biobb_amber/raw/master/biobb_amber/test/data/sander/sander.rst>`_. Accepted formats: rst (edam:format_3886), rst7 (edam:format_3886), netcdf (edam:format_3650), nc (edam:format_3650), ncrst (edam:format_3650).
+        input_ref_path (str) (Optional): Input reference coordinates for position restraints. File type: input. `Sample file <https://github.com/bioexcel/biobb_amber/raw/master/biobb_amber/test/data/sander/sander.rst>`_. Accepted formats: rst (edam:format_3886), rst7 (edam:format_3886), netcdf (edam:format_3650), nc (edam:format_3650), ncrst (edam:format_3886), crd (edam:format_3878).
         output_log_path (str): Output log file. File type: output. `Sample file <https://github.com/bioexcel/biobb_amber/raw/master/biobb_amber/test/reference/sander/sander.log>`_. Accepted formats: log (edam:format_2330), out (edam:format_2330), txt (edam:format_2330), o (edam:format_2330).
         output_traj_path (str): Output trajectory file. File type: output. `Sample file <https://github.com/bioexcel/biobb_amber/raw/master/biobb_amber/test/reference/sander/sander.x>`_. Accepted formats: trj (edam:format_3878), crd (edam:format_3878), mdcrd (edam:format_3878), x (edam:format_3878), netcdf (edam:format_3650), nc (edam:format_3650).
-        output_rst_path (str): Output restart file. File type: output. `Sample file <https://github.com/bioexcel/biobb_amber/raw/master/biobb_amber/test/reference/sander/sander.rst>`_. Accepted formats: rst (edam:format_3886), rst7 (edam:format_3886), netcdf (edam:format_3650), nc (edam:format_3650), ncrst (edam:format_3650).
+        output_rst_path (str): Output restart file. File type: output. `Sample file <https://github.com/bioexcel/biobb_amber/raw/master/biobb_amber/test/reference/sander/sander.rst>`_. Accepted formats: rst (edam:format_3886), rst7 (edam:format_3886), netcdf (edam:format_3650), nc (edam:format_3650), ncrst (edam:format_3886).
         output_cpout_path (str) (Optional): Output constant pH file (AMBER cpout). File type: output. `Sample file <https://github.com/bioexcel/biobb_amber/raw/master/biobb_amber/test/reference/sander/sander.cpout>`_. Accepted formats: cpout (edam:format_2330).
         output_cprst_path (str) (Optional): Output constant pH restart file (AMBER rstout). File type: output. `Sample file <https://github.com/bioexcel/biobb_amber/raw/master/biobb_amber/test/reference/sander/sander.cprst>`_. Accepted formats: cprst (edam:format_3886), rst (edam:format_3886), rst7 (edam:format_3886).
         output_mdinfo_path (str) (Optional): Output MD info. File type: output. `Sample file <https://github.com/bioexcel/biobb_amber/raw/master/biobb_amber/test/reference/sander/sander.mdinfo>`_. Accepted formats: mdinfo (edam:format_2330).
@@ -69,8 +69,12 @@ class SanderMDRun():
     def __init__(self, input_top_path: str, input_crd_path: str, output_log_path: str, output_traj_path: str, output_rst_path: str,
                  input_ref_path: str = None, input_mdin_path: str = None, input_cpin_path: str = None, output_cpout_path: str = None, output_cprst_path: str = None, output_mdinfo_path: str = None,
                  properties: dict = None, **kwargs) -> None:
+
         properties = properties or {}
 
+        # Call parent class constructor
+        super().__init__(properties)
+
         # Input/Output files
         self.io_dict = {
             'in': { 'input_top_path': input_top_path,
@@ -92,21 +96,18 @@ class SanderMDRun():
         self.sander_path = properties.get('sander_path', "sander")
         self.mdin = {k: str(v) for k, v in properties.get('mdin', dict()).items()}
 
+        if 'restraintmask' in self.mdin and self.mdin['restraintmask'][0] != '"' and self.mdin['restraintmask'][-1] != '"':
+            self.mdin['restraintmask'] = "\"" + self.mdin['restraintmask'] + "\""
+
         # Properties for MPI
         self.mpi_bin = properties.get('mpi_bin')
         self.mpi_np = properties.get('mpi_np')
         self.mpi_flags = properties.get('mpi_flags')
 
-        # Properties common in all BB
-        self.can_write_console_log = properties.get('can_write_console_log', True)
-        self.global_log = properties.get('global_log', None)
-        self.prefix = properties.get('prefix', None)
-        self.step = properties.get('step', None)
-        self.path = properties.get('path', '')
-        self.remove_tmp = properties.get('remove_tmp', True)
-        self.restart = properties.get('restart', False)
+        # Check the properties
+        self.check_properties(properties)
 
-    def check_data_params(self, out_log):
+    def check_data_params(self, out_log, out_err):
         """ Checks input/output paths correctness """
 
         # Check input(s)
@@ -276,27 +277,16 @@ class SanderMDRun():
     def launch(self):
         """Launches the execution of the BuildLinearStructure module."""
 
-        # Get local loggers from launchlogger decorator
-        out_log = getattr(self, 'out_log', None)
-        err_log = getattr(self, 'err_log', None)
-
         # check input/output paths and parameters
-        self.check_data_params(out_log)
+        self.check_data_params(self.out_log, self.err_log)
 
-        # Check the properties
-        fu.check_properties(self, self.properties)
-
-        # Restart
-        if self.restart:
-            output_file_list = [self.io_dict['out']['output_traj_path'],
-                                self.io_dict['out']['output_rst_path']]
-            if fu.check_complete_files(output_file_list):
-                fu.log('Restart is enabled, this step: %s will the skipped' % self.step, out_log, self.global_log)
-                return 0
+        # Setup Biobb
+        if self.check_restart(): return 0
+        self.stage_files()
 
         # Creating temporary folder
         self.tmp_folder = fu.create_unique_dir()
-        fu.log('Creating %s temporary folder' % self.tmp_folder, out_log)
+        fu.log('Creating %s temporary folder' % self.tmp_folder, self.out_log)
 
         #if self.io_dict['in']['input_mdin_path']:
         #    self.output_mdin_path = self.io_dict['in']['input_mdin_path']
@@ -306,7 +296,7 @@ class SanderMDRun():
 
         # Command line
         # sander -O -i mdin/min.mdin -p $1.cpH.prmtop -c ph$i/$1.inpcrd -r ph$i/$1.min.rst7 -o ph$i/$1.min.o
-        cmd = [self.sander_path,
+        self.cmd = [self.sander_path,
                '-O',
                '-i', self.output_mdin_path,
                '-p', self.io_dict['in']['input_top_path'],
@@ -317,24 +307,24 @@ class SanderMDRun():
                ]
 
         if self.io_dict['in']['input_ref_path']:
-            cmd.append('-ref')
-            cmd.append(self.io_dict['in']['input_ref_path'])
+            self.cmd.append('-ref')
+            self.cmd.append(self.io_dict['in']['input_ref_path'])
 
         if self.io_dict['in']['input_cpin_path']:
-            cmd.append('-cpin')
-            cmd.append(self.io_dict['in']['input_cpin_path'])
+            self.cmd.append('-cpin')
+            self.cmd.append(self.io_dict['in']['input_cpin_path'])
 
         if self.io_dict['out']['output_mdinfo_path']:
-            cmd.append('-inf')
-            cmd.append(self.io_dict['out']['output_mdinfo_path'])
+            self.cmd.append('-inf')
+            self.cmd.append(self.io_dict['out']['output_mdinfo_path'])
 
         if self.io_dict['out']['output_cpout_path']:
-            cmd.append('-cpout')
-            cmd.append(self.io_dict['out']['output_cpout_path'])
+            self.cmd.append('-cpout')
+            self.cmd.append(self.io_dict['out']['output_cpout_path'])
 
         if self.io_dict['out']['output_cprst_path']:
-            cmd.append('-cprestrt')
-            cmd.append(self.io_dict['out']['output_cprst_path'])
+            self.cmd.append('-cprestrt')
+            self.cmd.append(self.io_dict['out']['output_cprst_path'])
 
         # general mpi properties
         if self.mpi_bin:
@@ -344,20 +334,21 @@ class SanderMDRun():
                 mpi_cmd.append(str(self.mpi_np))
             if self.mpi_flags:
                 mpi_cmd.extend(self.mpi_flags)
-            cmd = mpi_cmd + cmd
+            self.cmd = mpi_cmd + cmd
 
-        fu.log('Creating command line with instructions and required arguments', out_log, self.global_log)
+        # Run Biobb block
+        self.run_biobb()
 
-        # Launch execution
-        returncode = cmd_wrapper.CmdWrapper(cmd, out_log, err_log, self.global_log).launch()
+        # Copy files to host
+        self.copy_to_host()
 
-        # Remove temporary file(s)
+        # remove temporary folder(s)
         if self.remove_tmp:
-            fu.rm(self.tmp_folder)
-            fu.rm("mdinfo")
-            fu.log('Removed: mdinfo, %s' % str(self.tmp_folder), out_log)
+            self.tmp_files.append(self.tmp_folder)
+            self.tmp_files.append("mdinfo")
+            self.remove_tmp_files()
 
-        return returncode
+        return self.return_code
 
 def sander_mdrun(input_top_path: str, input_crd_path: str,
             output_log_path: str, output_traj_path: str, output_rst_path: str,
